//
// UR5FFR Si5351 NWT
// v1.3
// Copyright (c) Andrey Belokon, 2017,2018 Odessa 
// https://github.com/andrey-belokon/
// GNU GPL license
//

#include "i2c.h"
#include "si5351a.h"

/*
  I2C address mapping
  I2C device found at address 0x60  ! Si5351
*/

// уровень сигнала на выходе Si5351. 0=2mA, 1=4mA, 2=6mA, 3=8mA
/*
 * CLK0 - Out
 * CLK1 - VFO
 * CLK2 - SA612
 */
#define SI5351_CLK0_DRIVE   0
#define SI5351_CLK1_DRIVE   3
#define SI5351_CLK2_DRIVE   0

#define SI5351_CALIBRATION       25003566

#define IF_FREQ       455000
#define TRIPLE_FREQ   145000000
#define MAX_FREQ      150000000

#define FIRMWARE_VERSION    "1.3"

Si5351 vfo5351;

char *ReadNumber(char *p, long *value)
{
  long val = 0;
  char c;
  //if (negate = (*p == '-')) p++;
  while ((c = *p++) && isdigit(c)) val = val*10+c-'0';
  *value = val;//(negate ? -val : val);
  return p;
}

char *SkipWhitespace(char *p){
  while (*p && (*p == ' ' || *p == ','))
    p++;
  return p;
} 

char cmd_buf[100];
int icmd_buf = 0;
byte mode = 0;
long det_freq = 0;
long freq;
byte vfo_mode = 0;
long vfo_freq = 0;

/*
 * mode
 * 0 = wide
 * 1 - narrow same (det_freq == 0)
 * 2 - narrow const (det_freq)
 */

bool UpdateFreq()
{
/*
 * CLK0 - Out
 * CLK1 - VFO
 * CLK2 - SA612
 */
  long fvfo=0;
  switch (vfo_mode) {
    case 0:
      fvfo = 0;
      break;
    case 1:
      fvfo = vfo_freq;
      break;
    case 2:
      fvfo = freq+vfo_freq;
      break;
  }
  if (fvfo > MAX_FREQ) fvfo = 0;
  if (freq < TRIPLE_FREQ) {
    switch (mode) {
      case 0:
      case 1:
        return vfo5351.set_freq(
          freq,
          fvfo,
          (det_freq == 0 ? freq+IF_FREQ : det_freq+IF_FREQ)
        );
        break;
      case 2:
        return vfo5351.set_freq(
          freq,
          fvfo,
          0
        );
        break;
    }
  } else {
    switch (mode) {
      case 0:
      case 1:
        return vfo5351.set_freq(
          freq/3,
          fvfo,
          (det_freq == 0 ? (freq+IF_FREQ)/3 : det_freq+IF_FREQ)
        );
        break;
      case 2:
        return vfo5351.set_freq(
          freq/3,
          fvfo,
          0
        );
        break;
    }
  }
  return false;
}

bool IsFreqOK()
{
/*
 * CLK0 - Out
 * CLK1 - VFO
 * CLK2 - SA612
 */
  bool ok=false;
  switch (mode) {
    case 0:
    case 1:
      ok = vfo5351.is_freq_ok(0) && vfo5351.is_freq_ok(2);
      break;
    case 2:
      ok = vfo5351.is_freq_ok(0);
      break;
  }
  if (vfo_mode != 0)
    ok = ok && vfo5351.is_freq_ok(1);
  return ok;
}

void ParseCommand()
{
  char *p = cmd_buf;
  char cmd = *p++;
  long step,count,xfreq;
  int val;
  p = SkipWhitespace(p);
  switch (cmd) {
    case 'w':
      // get firmware version
      Serial.println(FIRMWARE_VERSION);
      break;
    case 'c':
      // калибровка частоты сишки
      vfo5351.set_freq(5000000,5000000,5000000);
      vfo5351.out_calibrate_freq();
      break;
    case 'x':
      // задать калибровочную частоту
      // x 25003566
      ReadNumber(p,&xfreq);
      vfo5351.set_xtal_freq(xfreq);
      break;
    case 'o':
      // off. отключить генерацию
      vfo5351.set_freq(0,0,0);
      break;
    case 'm':
      /*
       * mode
       * 0 = narrow 50R
       * 1 - narrow HiZ
       * 2 - wide
       */
      mode = *p++ - '0';
      digitalWrite(3, mode == 1);
      digitalWrite(6, mode == 2);
      break;
    case 'n':
       // narrow frew
       // n det_freq
      p = SkipWhitespace(p);
      if (*p) ReadNumber(p,&det_freq);
      else det_freq=0;
      break;
    case 'f':
      // установить частоту
      // f 5000000
      ReadNumber(p,&freq);
      if (IsFreqOK())
        UpdateFreq();
      else {
        UpdateFreq();
        delay(100);
        analogRead(A0);
      }
      break;
    case 'v':
      // установить частоту VFO
      // v 0 - выключить
      // v 1 9100000 - fixed
      // v 2 9100000 - shift
      vfo_mode = *p++ - '0';
      if (vfo_mode != 0) {
        p = SkipWhitespace(p);
        ReadNumber(p,&vfo_freq);
      }
      UpdateFreq();
      break;
    case 'r':
      // read data
      // r step count
      p = ReadNumber(p,&step);
      p = SkipWhitespace(p);
      ReadNumber(p,&count);
      while (count-- > 0) {
        // при меньших значениях не успевает установиться частота на выходе
        /*if (UpdateFreq()) delay(10); 
        else delay(5);*/
        UpdateFreq();
        if (IsFreqOK()) {
          delay(10);
          val = analogRead(A0);
          val += analogRead(A0);
          val += analogRead(A0);
          val += analogRead(A0);
          val += analogRead(A0);
        } else {
          val = 0;
        }
        Serial.println(val);
        freq += step;
      }
      break;
  }
  icmd_buf = 0;
}

void loop()
{
  if (Serial.available()) {
    int inbyte = Serial.read();
  
    if (inbyte == 13){
      cmd_buf[icmd_buf] = 0;
      icmd_buf = 0;
      ParseCommand();
    } else if (icmd_buf < int(sizeof(cmd_buf))-1) {
      cmd_buf[icmd_buf++] = inbyte;
    }
  }
}

void setup()
{
  Serial.begin(115200);
  Serial.println(FIRMWARE_VERSION);
  i2c_init();
  // change for required output level
  vfo5351.setup(
    SI5351_CLK0_DRIVE,
    SI5351_CLK1_DRIVE,
    SI5351_CLK2_DRIVE
  ); 
  vfo5351.set_xtal_freq(SI5351_CALIBRATION);
  analogReference(INTERNAL);
  pinMode(A0,INPUT);
  pinMode(3,OUTPUT);
  pinMode(6,OUTPUT);
  digitalWrite(3,0);
  digitalWrite(6,0);
  //freq = 14000000;
  //UpdateFreq();
}
